import os
from requests.auth import HTTPDigestAuth
from requests import Session
from .tools import LiveServerSession




HIKVISION_ACT_LOGIN = os.environ.get('HIKVISION_ACT_LOGIN')
HIKVISION_ACT_PASSWORD = os.environ.get('HIKVISION_ACT_PASSWORD')




class LoginPasswordMissingError(Exception):
    pass


if HIKVISION_ACT_LOGIN is None or HIKVISION_ACT_PASSWORD is None:
    raise LoginPasswordMissingError(
        "All Hikvision Access Control Terminals API methods require login and password!"
    )

auth = HTTPDigestAuth(HIKVISION_ACT_LOGIN, HIKVISION_ACT_PASSWORD)
session = Session()

session.auth = auth


from .person import Person
from .fplib import FaceData, FaceDataLib
from .event import Event


from . import session
import json
from types import SimpleNamespace


class FaceDataLib(object):

    def fp_library_add(self, faceLibType, name, customInfo, host):
        path = host+'/ISAPI/Intelligent/FDLib?format=json'
        body = {
            'faceLibType': faceLibType,
            'name': name,
            'customInfo': customInfo
        }
        response = session.post(path, data=json.dumps(body))
        

        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def fp_library_update(self, fdid, faceLibType, name, customInfo, host):
        path = f'{host}/ISAPI/Intelligent/FDLib?format=json&FDID={fdid}&faceLibType={faceLibType}'
        body = {
            "name": "CustomTestLibraryBlackFD",
            "customInfo": "test libraryBlackFD"
            }
        response = session.put(path, data=json.dumps(body))
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def fp_library_delete(self, fdid, faceLibType, host):
        path = f'{host}/ISAPI/Intelligent/FDLib?format=json&FDID={fdid}&faceLibType={faceLibType}'
        
        response = session.delete(path)
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def fp_library_list(self, host):
        path = '{host}/ISAPI/Intelligent/FDLib?format=json'
        response = session.get(path)
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result


class FaceData(object):
    
    def face_data_add(self, faceLibType, FDID, FPID, name, gender, bornTime, city, faceURL, host):
        path = host+'/ISAPI/Intelligent/FDLib/FaceDataRecord?format=json'
        body = {
            "faceLibType": faceLibType,
            "FDID": str(FDID),
            "FPID": str(FPID),
            "name": name,
            "gender": gender,
            "bornTime": bornTime, #"19940226T000000+0500"
            "city": city,
            "faceURL": faceURL
            }
        response = session.post(path, data=json.dumps(body))
        
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def face_data_update(self, faceLibType, FDID, FPID, name, gender, bornTime, city, faceURL, host):
        path = f'{host}/ISAPI/Intelligent/FDLib/FDSearch?format=json&FDID={FDID}&FPID={FPID}&faceLibType={faceLibType}'
        body = {
            "name": name,
            "gender": gender,
            "bornTime": bornTime, #"19940226T000000+0500"
            "city": city,
            "faceURL": faceURL
            }
        response = session.put(path, data=json.dumps(body))
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def face_data_delete(self, faceLibType, FDID, FPIDList, host):
        path = f'{host}/ISAPI/Intelligent/FDLib/FDSearch/Delete?format=json&FDID={FDID}&faceLibType={faceLibType}'
        fpidlist = []
        for fpid in FPIDList:
            fpidlist.append({
                'value': fpid
            })
        body = {
            'FPID': fpidlist
            }
             
        response = session.put(path, data=json.dumps(body))
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def face_data_search(self, faceLibType, FDID, FPID, host):
        path = f'{host}/ISAPI/Intelligent/FDLib/FDSearch?format=json'
        body = {
            "searchResultPosition": 0,
            "maxResults": 32,
            "faceLibType": f'{faceLibType}',
            "FDID": f'{FDID}',
            "FPID": f'{FPID}'
            }
        response = session.post(path, data=json.dumps(body))
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result


from . import session
import json
from types import SimpleNamespace


class Person(object):
    def __init__(self):
        pass

    def search(self, id, host):
        path = host+'/ISAPI/AccessControl/UserInfo/Search?format=json'
        body = {
                    "UserInfoSearchCond": {
                        "searchID": "4",
                        "searchResultPosition": 0,
                        "maxResults": 32,
                        "EmployeeNoList":[
                            {
                                "employeeNo": str(id)
                            }
                        ]
                    }
                }
        response = session.post(path, data=json.dumps(body))
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def add(self, id, name, user_type, password, gender, doors, host):
        path = host+'/ISAPI/AccessControl/UserInfo/Record?format=json'
        body = {
                    "UserInfo":
                        {
                            "employeeNo":str(id),
                            "name": name,
                            "userType": user_type,
                            "Valid":{
                                "enable": False,
                                "beginTime":"2017-08-01T17:30:08",
                                "endTime":"2022-08-01T17:30:08",
                                "timeType":"local"
                                },
                            "doorRight": doors,
                            "RightPlan": [
                                {
                                    "doorNo": 1,
                                    "planTemplateNo": "1"
                                }
                            ],
                            
                            "password":password,
                            
                            "gender":gender
                        }
                }
        response = session.post(path, data=json.dumps(body))
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result
    
    def delete(self, id, host):
        path = host+'/ISAPI/AccessControl/UserInfo/Delete?format=json'
        body = {
                    "UserInfoDelCond": {                        
                        "EmployeeNoList":[
                            {
                                "employeeNo": str(id)
                            }
                        ]
                    }
                }
        response = session.put(path, data=json.dumps(body))
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def update(self, id, name, user_type, password, gender, doors, host):
        path = host+'/ISAPI/AccessControl/UserInfo/Modify?format=json'
        body = {
                    "UserInfo":
                        {
                            "employeeNo":str(id),
                            "name": name,
                            "userType": user_type,
                            "Valid":{
                                "enable": False,
                                "beginTime":"2017-08-01T17:30:08",
                                "endTime":"2022-08-01T17:30:08",
                                "timeType":"local"
                                },
                            "doorRight": doors,
                            "RightPlan": [
                                {
                                    "doorNo": 1,
                                    "planTemplateNo": "1"
                                }
                            ],
                            
                            "password":password,
                            
                            "gender":gender
                        }
                }
        response = session.put(path, data=json.dumps(body))
        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result

    def get_count(self, host):
        path = host+'/ISAPI/AccessControl/UserInfo/Count?format=json'
        
        response = session.get(path)        
        result = json.loads(json.dumps(response.json()), object_hook=lambda d: SimpleNamespace(**d))
        return result.UserInfoCount.userNumber



from . import session
import json
import threading


class Event(object):
    
    def __init__(self):
        self._stop = False    
    
    def start_listen_events(self, _callback):
        self._callback = _callback
        x = threading.Thread(target=self._start_listen_events,)        
        x.start()

    def stop_listen_events(self):
        self._stop = True
    
    def get_status(self):
        return not self._stop

    def _start_listen_events(self):
        path = '/ISAPI/Event/notification/alertStream'
        response = session.get(path, stream=True)
        response.raise_for_status()
        
        in_header = False             # are we parsing headers at the moment
        grabbing_response = False     # are we grabbing the response at the moment
        response_size = 0             # the response size that we take from Content-Length
        response_buffer = b""         # where we keep the reponse bytes

        for chunk in response.iter_lines():
            decoded = ""
            try:
                decoded = chunk.decode("utf-8")
            except:
                # image bytes here ignore them
                continue

            if decoded == "--MIME_boundary":                
                in_header = True

            if in_header:
                if decoded.startswith("Content-Length"):
                    decoded.replace(" ", "")
                    content_length = decoded.split(":")[1]
                    response_size = int(content_length)

                if decoded == "":
                    in_header = False
                    grabbing_response = True

            elif grabbing_response:
                response_buffer += chunk

                if len(response_buffer) != response_size:
                    response_buffer += b"\n"
                else:
                    # time to convert it json and return it
                    grabbing_response = False
                    print(response_buffer)
                    dic = json.loads(response_buffer)
                    
                    if dic["eventType"] == "AccessControllerEvent":
                        rsp = {
                            "date": dic["dateTime"],
                            "status": dic["AccessControllerEvent"]["attendanceStatus"],
                        }
                        if rsp["status"] == "checkIn":
                            rsp["employee_id"] = int(dic["AccessControllerEvent"]["employeeNoString"])
                        self._callback(rsp)
                    response_buffer = b""
                    if self._stop:
                        return 0



class CreateFPLibResponse(object):
    
    def __init__(self, requestURL, statusCode, statusString, subStatusCode, errorCode, errorMsg, FDID):
        self.requestURL = requestURL
        self.statusCode = statusCode
        self.statusString = statusString
        self.subStatusCode = subStatusCode
        self.errorCode = errorCode
        self.errorMsg = errorMsg
        self.FDID = FDID


from distutils.core import setup
setup(
  name = 'hikvision_isapi_wrapper',
  packages = ['hikvision_isapi_wrapper'],
  version = '0.3.3',
  license='MIT',
  description = 'Minimial API wrapper for hikvison ISAPI',
  author = 'shadowwa1k3r',
  author_email = 'ergash1994@gmail.com',
  url = 'https://github.com/shadowwa1k3r/hikvision-isapi-wrapper',
  download_url = 'https://github.com/shadowwa1k3r/hikvision-isapi-wrapper/archive/v_033.tar.gz',
  keywords = ['hikvision', 'ISAPI', 'wrapper'],
  install_requires=[
          'atomicwrites==1.4.0',
          'attrs==20.3.0',
          'certifi==2020.12.5',
          'chardet==4.0.0',
          'colorama==0.4.4',
          'idna==2.10',
          'iniconfig==1.1.1',
          'multidict==5.1.0',
          'packaging==20.8',
          'pluggy==0.13.1',
          'py==1.10.0',
          'pyparsing==2.4.7',
          'pytest==6.2.1',
          'pyyaml==5.3.1',
          'requests==2.25.1',
          'six==1.15.0',
          'toml==0.10.2',
          'urllib3==1.26.2',
          'vcrpy==4.1.1',
          'wrapt==1.12.1',
          'yarl==1.6.3',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
)